| Safe Haskell | None |
|---|---|
| Language | GHC2021 |
Control.Monad.Borrow.Pure.Affine.Unsafe
Contents
Description
This module provides unsafe internals of Control.Monad.Borrow.Pure.Affine. These are not meant to be used by end-users, so generally YOU SHOULD NOT import this module, and import Control.Monad.Borrow.Pure.Affine instead.
This module is meant for library authors who want to build a new API on top of Pure Borrow.
This module provides internals of BO and Alias, which can break the soundness guarded by the role system.
We STRONGLY recommend to you to import only the needed parts of the definitions, and not to import everything or qualified.